#!/usr/local/bin/swig -python -o gtkmodule.c
#
# SWIG wrapper for Gtk-Python

%module gtk

%{
#include <gtk/gtk.h>
%}

%include gtkobject.i
%include obj_tmpl.i
%include gdk_typemap.i
%include gtk_typemap.i

// signal handler
%include gtksignal.i

// main loop handler and rc file parsing.
%include gtkmain.i

// gtk object general new/set/get routines
%include gtknewset.i

// some useful gdk functions
%include gdkfuncs.i

char *gtk_type_name(int tp);
int gtk_type_from_name(char *name);

int GTK_IS_OBJECT(GtkObject *o);
int GTK_OBJECT_FLAGS(GtkObject *o);
int GTK_OBJECT_TYPE(GtkObject *o);
void GTK_OBJECT_SET_FLAGS(GtkObject *o, int flags);
void GTK_OBJECT_UNSET_FLAGS(GtkObject *o, int flags);
%{
static PyObject *gtk__object_set_data(PyObject *self, PyObject *args) {
	PyGtk_Object *obj;
	char *key;
	PyObject *data;
	if (!PyArg_ParseTuple(args, "O!sO:gtk_object_set_data", &PyGtk_Type,
			&obj, &key, &data))
		return NULL;
	Py_INCREF(data);
	gtk_object_set_data_full(PyGtk_Get(obj), key, data,
		(GtkDestroyNotify)PyGtk_DestroyNotify);
	Py_INCREF(Py_None);
	return Py_None;
}
static PyObject *gtk__object_get_data(PyObject *self, PyObject *args) {
	PyGtk_Object *obj;
	char *key;
	PyObject *data;
	if (!PyArg_ParseTuple(args, "O!s:gtk_object_get_data", &PyGtk_Type,
			&obj, &key))
		return NULL;
	data = gtk_object_get_data(PyGtk_Get(obj), key);
	if (data != NULL) {
		Py_INCREF(data);
		return data;
	} else {
		Py_INCREF(Py_None);
		return Py_None;
	}
}
%}
%native(gtk_object_set_data) PyObject *gtk__object_set_data(PyObject *self,
	PyObject *args);
%native(gtk_object_get_data) PyObject *gtk__object_get_data(PyObject *self,
	PyObject *args);
void gtk_object_remove_data(GtkObject *o, char *key);
int GTK_IS_WIDGET(GtkObject *o);
void gtk_widget_destroy(GtkWidget *w);
void gtk_widget_unparent(GtkWidget *w);
void gtk_widget_show(GtkWidget *w);
void gtk_widget_hide(GtkWidget *w);
void gtk_widget_show_all(GtkWidget *w);
void gtk_widget_show_now(GtkWidget *w);
void gtk_widget_hide_all(GtkWidget *w);
void gtk_widget_map(GtkWidget *w);
void gtk_widget_unmap(GtkWidget *w);
void gtk_widget_realize(GtkWidget *w);
void gtk_widget_unrealize(GtkWidget *w);
void gtk_widget_queue_draw(GtkWidget *w);
void gtk_widget_queue_resize(GtkWidget *w);
void gtk_widget_draw(GtkWidget *w, GdkRectangle *a);
void gtk_widget_draw_focus(GtkWidget *w);
void gtk_widget_draw_default(GtkWidget *w);
void gtk_widget_size_request(GtkWidget *w, GtkRequisition *r);
void gtk_widget_size_allocate(GtkWidget *w, GtkAllocation *a);
void gtk_widget_install_accelerator(GtkWidget *w, GtkAcceleratorTable *t,
	char *signame, char key, int modifiers);
void gtk_widget_remove_accelerator(GtkWidget *w, GtkAcceleratorTable *t,
	char *signame);
// gtk_widget_event
void gtk_widget_activate(GtkWidget *w);
void gtk_widget_reparent(GtkWidget *w, GtkWidget *p);
void gtk_widget_popup(GtkWidget *w, int x, int y);
void gtk_widget_intersect(GtkWidget *w, GdkRectangle *a, GdkRectangle *out);
void gtk_widget_basic(GtkWidget *w);
void gtk_widget_grab_focus(GtkWidget *w);
void gtk_widget_grab_default(GtkWidget *w);
// void gtk_widget_restore_state(GtkWidget *w);
void gtk_widget_set_name(GtkWidget *w, char *name);
char *gtk_widget_get_name(GtkWidget *w);
void gtk_widget_set_state(GtkWidget *w, int state);
void gtk_widget_set_sensitive(GtkWidget *w, int s);
void gtk_widget_set_parent(GtkWidget *w, GtkWidget *p);
// gtk_widget_set_parent_window
// gtk_widget_get_parent_window
void gtk_widget_set_style(GtkWidget *w, GtkStyle *s);
void gtk_widget_set_rc_style(GtkWidget *w);
void gtk_widget_ensure_style(GtkWidget *w);
void gtk_widget_restore_default_style(GtkWidget *w);
void gtk_widget_propagate_default_style();
void gtk_widget_set_uposition(GtkWidget *w, int x, int y);
void gtk_widget_set_usize(GtkWidget *w, int w, int h);
void gtk_widget_set_events(GtkWidget *w, int e);
void gtk_widget_set_extension_events(GtkWidget *w, int mode);
GtkWidget *gtk_widget_get_toplevel(GtkWidget *w);
GtkWidget *gtk_widget_get_ancestor(GtkWidget *w, int t);
// gtk_widget_get_colormap
// gtk_widget_get_visual
GtkStyle *gtk_widget_get_style(GtkWidget *w);
int gtk_widget_get_events(GtkWidget *w);
int gtk_widget_get_extension_events(GtkWidget *w);
%{
static PyObject *gtk_widget__get_pointer(PyObject *self, PyObject *args) {
  int x, y;
  PyGtk_Object *w;
  if (!PyArg_ParseTuple(args, "O!:gtk_widget_get_pointer", &PyGtk_Type, &w))
    return NULL;
  gtk_widget_get_pointer(GTK_WIDGET(w->go), &x, &y);
  return Py_BuildValue("(ii)", x, y);
}
%}
%native(gtk_widget_get_pointer) PyObject *gtk_widget__get_pointer(PyObject
                                                    *self, PyObject *args);
int gtk_widget_is_ancestor(GtkWidget *w, GtkWidget *a);
int gtk_widget_is_child(GtkWidget *w, GtkWidget *c);
int gtk_widget_hide_on_delete(GtkWidget *w);

// gtk_widget_push_colormap
// gtk_widget_push_visual
void gtk_widget_push_style(GtkStyle *style);
// gtk_widget_pop_colormap
// gtk_widget_pop_visual
void gtk_widget_pop_style();
// gtk_widget_set_default_colormap
// gtk_widget_set_default_visual
void gtk_widget_set_default_style(GtkStyle *style);
// gtk_widget_get_default_colormap
// gtk_widget_get_default_visual
// gtk_widget_get_default_style
%{
static PyObject *
gtk__widget_dnd_drag_set(PyObject *self, PyObject *args) {
    PyGtk_Object *obj;
    PyObject *pylist, *item;
    char **list;
    int enable, numtypes, i;
    if (!PyArg_ParseTuple(args, "O!iO:gtk_widget_dnd_drag_set", &PyGtk_Type,
			  &obj, &enable, &pylist))
        return NULL;
    if (!PySequence_Check(pylist)) {
        PyErr_SetString(PyExc_TypeError, "3rd argument not a sequence");
	return NULL;
    }
    numtypes = PySequence_Length(pylist);
    list = malloc(numtypes * sizeof(char *));
    for (i = 0; i < numtypes; i++) {
        item = PySequence_GetItem(pylist, i);
        if (!PyString_Check(item)) {
	    PyErr_SetString(PyExc_TypeError, "sequence item not a string");
	    free(list);
	    return NULL;
	}
	list[i] = PyString_AsString(item);
    }
    gtk_widget_dnd_drag_set(GTK_WIDGET(PyGtk_Get(obj)), enable, list,numtypes);
    free(list);
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
gtk__widget_dnd_drop_set(PyObject *self, PyObject *args) {
    PyGtk_Object *obj;
    PyObject *pylist, *item;
    char **list;
    int enable, is_destructive, numtypes, i;
    if (!PyArg_ParseTuple(args, "O!iOi:gtk_widget_dnd_drop_set", &PyGtk_Type,
			  &obj, &enable, &pylist, &is_destructive))
        return NULL;
    if (!PySequence_Check(pylist)) {
        PyErr_SetString(PyExc_TypeError, "3rd argument not a sequence");
	return NULL;
    }
    numtypes = PySequence_Length(pylist);
    list = malloc(numtypes * sizeof(char *));
    for (i = 0; i < numtypes; i++) {
        item = PySequence_GetItem(pylist, i);
        if (!PyString_Check(item)) {
	    PyErr_SetString(PyExc_TypeError, "sequence item not a string");
	    free(list);
	    return NULL;
	}
	list[i] = PyString_AsString(item);
    }
    gtk_widget_dnd_drop_set(GTK_WIDGET(PyGtk_Get(obj)), enable, list,
			    numtypes, is_destructive);
    free(list);
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
gtk__widget_dnd_data_set(PyObject *self, PyObject *args) {
    PyGtk_Object *wid;
    PyGdkEvent_Object *event;
    char *string;
    int len;
    if (!PyArg_ParseTuple(args, "O!O!z#:gtk_widget_dnd_data_set", &PyGtk_Type,
			  &wid, &PyGdkEvent_Type, &event, &string, &len))
        return NULL;
    gtk_widget_dnd_data_set(GTK_WIDGET(PyGtk_Get(wid)), PyGdkEvent_Get(event),
			    (gpointer) string, len);
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
gtk__widget_shape_combine_mask(PyObject *self, PyObject *args) {
    PyGtk_Object *obj, *pix;
    int x, y;
    GdkBitmap *mask;
    if (!PyArg_ParseTuple(args, "O!O!ii:gtk_widget_shape_combine_mask",
			  &PyGtk_Type, &obj, &PyGtk_Type, &pix, &x, &y))
        return NULL;
    mask = gdk_bitmap_ref(GTK_PIXMAP(PyGtk_Get(pix))->mask);
    gtk_widget_shape_combine_mask(GTK_WIDGET(PyGtk_Get(obj)), mask, x, y);
    Py_INCREF(Py_None);
    return Py_None;
}
%}
void gtk_widget_dnd_drag_add(GtkWidget *widget);
%native(gtk_widget_dnd_drag_set) PyObject *gtk__widget_dnd_drag_set(
						    PyObject *s, PyObject *a);
%native(gtk_widget_dnd_drop_set) PyObject *gtk__widget_dnd_drop_set(
						    PyObject *s, PyObject *a);
%native(gtk_widget_dnd_data_set) PyObject *gtk__widget_dnd_data_set(
						    PyObject *s, PyObject *a);
%native(gtk_widget_shape_combine_mask) PyObject *
  gtk__widget_shape_combine_mask(PyObject *self, PyObject *args);

%{
#define gtk_adjustment_get_lower(o)          ((o)->lower)
#define gtk_adjustment_get_upper(o)          ((o)->upper)
#define gtk_adjustment_get_value(o)          ((o)->value)
#define gtk_adjustment_get_step_increment(o) ((o)->step_increment)
#define gtk_adjustment_get_page_increment(o) ((o)->page_increment)
#define gtk_adjustment_get_page_size(o)      ((o)->page_size)
%}
int GTK_IS_ADJUSTMENT(GtkObject *o);
GtkObject *gtk_adjustment_new(double v, double l, double u, double si,
                                                 double pi, double ps);
double gtk_adjustment_get_lower(GtkAdjustment *a);
double gtk_adjustment_get_upper(GtkAdjustment *a);
double gtk_adjustment_get_value(GtkAdjustment *a);
double gtk_adjustment_get_step_increment(GtkAdjustment *a);
double gtk_adjustment_get_page_increment(GtkAdjustment *a);
double gtk_adjustment_get_page_size(GtkAdjustment *a);
void gtk_adjustment_set_value(GtkAdjustment *a, double value);
int GTK_IS_ALIGNMENT(GtkObject *o);
GtkWidget *gtk_alignment_new(double xa, double ya, double xs, double ys);
void gtk_alignment_set(GtkAlignment *a, double xa, double ya, double xs,
                                                              double ys);
int GTK_IS_ARROW(GtkObject *o);
GtkWidget *gtk_arrow_new(int at, int st);
void gtk_arrow_set(GtkArrow *a, int at, int st);
int GTK_IS_ASPECT_FRAME(GtkObject *o);
GtkWidget *gtk_aspect_frame_new(char *label, double xa, double ya,
                                     double ratio, int obey_child);
void gtk_aspect_frame_set(GtkAspectFrame *a, double xa, double ya,
                                     double ratio, int obey_child);
int GTK_IS_BIN(GtkObject *o);
int GTK_IS_BOX(GtkObject *o);
void gtk_box_pack_start(GtkBox *b, GtkWidget *w, int e, int f, int p);
void gtk_box_pack_end(GtkBox *b, GtkWidget *w, int e, int f, int p);
void gtk_box_pack_start_defaults(GtkBox *b, GtkWidget *w);
void gtk_box_pack_end_defaults(GtkBox *b, GtkWidget *w);
void gtk_box_set_homogeneous(GtkBox *b, int h);
void gtk_box_set_spacing(GtkBox *b, int s);
void gtk_box_reorder_child(GtkBox *b, GtkWidget *w, int pos);
int GTK_IS_BUTTON(GtkObject *o);
GtkWidget *gtk_button_new();
GtkWidget *gtk_button_new_with_label(char *l);
void gtk_button_pressed(GtkButton *b);
void gtk_button_released(GtkButton *b);
void gtk_button_enter(GtkButton *b);
void gtk_button_leave(GtkButton *b);
int GTK_IS_BUTTON_BOX(GtkObject *o);
%{
static PyObject *
gtk__button_box_get_child_size_default(PyObject *self, PyObject *args) {
    int mw, mh;
    if (!PyArg_ParseTuple(args, ":gtk_button_box_get_child_size_default"))
        return NULL;
    gtk_button_box_get_child_size_default(&mw, &mh);
    return Py_BuildValue("(ii)", mw, mh);
}
static PyObject *
gtk__button_box_get_child_ipadding_default(PyObject *self, PyObject *args) {
    int ipx, ipy;
    if (!PyArg_ParseTuple(args, ":gtk_button_box_get_child_ipadding_default"))
        return NULL;
    gtk_button_box_get_child_ipadding_default(&ipx, &ipy);
    return Py_BuildValue("(ii)", ipx, ipy);
}
static PyObject *
gtk__button_box_get_child_size(PyObject *self, PyObject *args) {
    int mw, mh;
    PyGtk_Object *o;
    if (!PyArg_ParseTuple(args, "O!:gtk_button_box_get_child_size",
            &PyGtk_Type, &o))
        return NULL;
    gtk_button_box_get_child_size(GTK_BUTTON_BOX(PyGtk_Get(o)), &mw, &mh);
    return Py_BuildValue("(ii)", mw, mh);
}
static PyObject *
gtk__button_box_get_child_ipadding_(PyObject *self, PyObject *args) {
    int ipx, ipy;
    PyGtk_Object *o;
    if (!PyArg_ParseTuple(args, "O!:gtk_button_box_get_child_ipadding",
            &PyGtk_Type, &o))
        return NULL;
    gtk_button_box_get_child_ipadding(GTK_BUTTON_BOX(PyGtk_Get(o)),&ipx,&ipy);
    return Py_BuildValue("(ii)", ipx, ipy);
}
%}
%native(gtk_button_box_get_child_size_default) PyObject *
    gtk__button_box_get_child_size_default(PyObject *self, PyObject *args);
%native(gtk_button_box_get_ipadding_default) PyObject *
    gtk__button_box_get_child_ipadding_default(PyObject *self, PyObject *args);
void gtk_button_box_set_child_size_default(int mw, int mh);
void gtk_button_box_set_child_ipadding_default(int ipx, int ipy);
int gtk_button_box_get_spacing(GtkButtonBox *w);
int gtk_button_box_get_layout(GtkButtonBox *w);
%native(gtk_button_box_get_child_size) PyObject *
    gtk__button_box_get_child_size(PyObject *self, PyObject *args);
%native(gtk_button_box_get_child_ipadding) PyObject *
    gtk__button_box_get_child_ipadding_(PyObject *self, PyObject *args);
void gtk_button_box_set_spacing(GtkButtonBox *w, int s);
void gtk_button_box_set_layout(GtkButtonBox *w, int l);
void gtk_button_box_set_child_size(GtkButtonBox *w, int mw, int mh);
void gtk_button_box_set_child_ipadding(GtkButtonBox *w, int ipx, int ipy);
int GTK_IS_CHECK_BUTTON(GtkObject *o);
GtkWidget *gtk_check_button_new();
GtkWidget *gtk_check_button_new_with_label(char *l);
%{
#define gtk_check_menu_item_get_active(cb) ((cb)->active)
%}
int GTK_IS_CHECK_MENU_ITEM(GtkObject *o);
GtkWidget *gtk_check_menu_item_new();
GtkWidget *gtk_check_menu_item_new_with_label(char *l);
void gtk_check_menu_item_set_state(GtkCheckMenuItem *c, int s);
void gtk_check_menu_item_toggled(GtkCheckMenuItem *c);
void gtk_check_menu_item_set_show_toggle(GtkCheckMenuItem *c, int always);
int gtk_check_menu_item_get_active(GtkCheckMenuItem *c);
int GTK_IS_CLIST(GtkObject *o);
GtkWidget *gtk_clist_new(int columns);
%{
static PyObject *gtk__clist_new_with_titles(PyObject *self, PyObject *args) {
	int c, i;
	PyObject *l, *item;
	char **list;
	if (!PyArg_ParseTuple(args, "iO:gtk_clist_new_with_titles",
			&c, &l))
		return NULL;
	if (!PySequence_Check(l)) {
		PyErr_SetString(PyExc_TypeError,"3rd argument not a sequence");
		return NULL;
	}
	if (PySequence_Length(l) < c) {
		PyErr_SetString(PyExc_TypeError, "sequence not long enough");
		return NULL;
	}
	list = malloc(sizeof(char *)*c);
	for (i = 0; i < c; i++) {
		item = PySequence_GetItem(l, i);
		if (!PyString_Check(item)) {
			PyErr_SetString(PyExc_TypeError,
				 "sequence item not a string");
			free(list);
			return NULL;
		}
		list[i] = PyString_AsString(item);
	}
	item = (PyObject *) PyGtk_New((GtkObject *)gtk_clist_new_with_titles
		(c, list));
	free(list);
	return item;
}
%}
%native(gtk_clist_new_with_titles) PyObject *gtk__clist_new_with_titles
		(PyObject *self, PyObject *args); 
int gtk_clist_row_is_visible(GtkCList *c, int row);
int gtk_clist_get_cell_type(GtkCList *c, int r, int c);
void gtk_clist_set_border(GtkCList *c, int border);
void gtk_clist_set_policy(GtkCList *c, int vsp, int hsp);
void gtk_clist_freeze(GtkCList *c);
void gtk_clist_thaw(GtkCList *c);
void gtk_clist_column_titles_show(GtkCList *c);
void gtk_clist_column_titles_hide(GtkCList *c);
void gtk_clist_column_title_active(GtkCList *c, int col);
void gtk_clist_column_title_passive(GtkCList *c, int col);
void gtk_clist_column_titles_active(GtkCList *c);
void gtk_clist_column_titles_passive(GtkCList *c);
void gtk_clist_set_column_title(GtkCList *c, int col, char *title);
void gtk_clist_set_column_widget(GtkCList *c, int col, GtkWidget *w);
void gtk_clist_set_column_justification(GtkCList *c, int col, int just);
void gtk_clist_set_column_width(GtkCList *c, int col, int width);
void gtk_clist_set_row_height(GtkCList *c, int height);
void gtk_clist_moveto(GtkCList *cl, int r, int c, double ra, double ca);
void gtk_clist_set_text(GtkCList *cl, int r, int c, char *text);
%{
static PyObject *gtk__clist_get_text(PyObject *self, PyObject *args) {
	PyGtk_Object *w;
	int r, c;
	char *text;
	if (!PyArg_ParseTuple(args, "O!ii:gtk_clist_get_text", &PyGtk_Type,
			&w, &r, &c))
		return NULL;
	gtk_clist_get_text(GTK_CLIST(PyGtk_Get(w)), r, c, &text);
	return PyString_FromString(text);
}
static PyObject *gtk__clist_set_pixmap(PyObject *self, PyObject *args) {
	PyGtk_Object *o, *w;
	int r, c;
	GtkWidget *wid;
	GdkPixmap *p;
	GdkBitmap *m;
	if (!PyArg_ParseTuple(args, "O!iiO!:gtk_clist_set_pixmap", &PyGtk_Type,
			&o, &r, &c, &PyGtk_Type, &w))
		return NULL;
	wid = GTK_WIDGET(PyGtk_Get(w));
	/* we do this so if the pixmap isn't used as a widget, it will be
           destroyed */
	gtk_object_sink(GTK_OBJECT(wid));
	if (!GTK_IS_PIXMAP(wid)) {
		PyErr_SetString(PyExc_TypeError, "4th arg must be a pixmap");
		return NULL;
	}
        /* clist_set_pixmap ref's the pixmap and mask, so we don't need to*/
	p = GTK_PIXMAP(wid)->pixmap;
	m = GTK_PIXMAP(wid)->mask;
	gtk_clist_set_pixmap(GTK_CLIST(PyGtk_Get(o)), r, c, p, m);
	Py_INCREF(Py_None);
	return Py_None;
}
static PyObject *gtk__clist_set_pixtext(PyObject *self, PyObject *args) {
	PyGtk_Object *o, *w;
	int r, c, spacing;
	char *text;
	GtkWidget *wid;
	GdkPixmap *p;
	GdkBitmap *m;
	if (!PyArg_ParseTuple(args, "O!iisiO!:gtk_clist_set_pixtext",
			&PyGtk_Type, &o, &r, &c, &text, &spacing,
			&PyGtk_Type, &w))
		return NULL;
	wid = GTK_WIDGET(PyGtk_Get(w));
	/* we do this so that if the pixmap is not show, it will be deleted */
	gtk_object_sink(GTK_OBJECT(wid));
	if (!GTK_IS_PIXMAP(wid)) {
		PyErr_SetString(PyExc_TypeError, "6th arg must be a pixmap");
		return NULL;
	}
        /* clist_set_pixtext ref's the pixmap and mask, so we don't need to*/
	p = GTK_PIXMAP(wid)->pixmap;
	m = GTK_PIXMAP(wid)->mask;
	gtk_clist_set_pixtext(GTK_CLIST(PyGtk_Get(o)), r, c, text, spacing,
		p, m);
	Py_INCREF(Py_None);
	return Py_None;
}
%}
%native(gtk_clist_get_text) PyObject *gtk__clist_get_text(
	PyObject *self, PyObject *args);;
%native(gtk_clist_set_pixmap) PyObject *gtk__clist_set_pixmap(
	PyObject *self, PyObject *args);
%native(gtk_clist_set_pixtext) PyObject *gtk__clist_set_pixtext(
	PyObject *self, PyObject *args);
// gtk_clist_set_pixmap
// gtk_clist_set_pixtext
// gtk_clist_get_pixmap
// gtk_clist_get_pixtext
void gtk_clist_set_foreground(GtkCList *cl, int row, GdkColor *c);
void gtk_clist_set_background(GtkCList *cl, int row, GdkColor *c);
void gtk_clist_set_selection_mode(GtkCList *cl, int mode);
void gtk_clist_set_shift(GtkCList *cl, int r, int c, int v, int h);
%{
static PyObject *gtk__clist_append(PyObject *self, PyObject *args) {
	PyGtk_Object *cl;
	int col, i;
	PyObject *l, *item;
	char **list;
	if (!PyArg_ParseTuple(args, "O!O:gtk_clist_append", &PyGtk_Type, &cl,
			&l))
		return NULL;
	if (!PySequence_Check(l)) {
		PyErr_SetString(PyExc_TypeError, "argument not a sequence");
		return NULL;
	}
	col = GTK_CLIST(PyGtk_Get(cl))->columns;
	if (PySequence_Length(l) < col) {
		PyErr_SetString(PyExc_TypeError, "sequnce too short");
		return NULL;
	}
	list = malloc(sizeof(char *) * col);
	for (i = 0; i < col; i++) {
		item = PySequence_GetItem(l, i);
		if (!PyString_Check(item)) {
			PyErr_SetString(PyExc_TypeError,
				"sequence item not a string");
			free(list);
			return NULL;
		}
		list[i] = PyString_AsString(item);
	}
	i = gtk_clist_append(GTK_CLIST(PyGtk_Get(cl)), list);
	free(list);
	return PyInt_FromLong(i);
}

static PyObject *gtk__clist_insert(PyObject *self, PyObject *args) {
	PyGtk_Object *cl;
	int col, r, i;
	PyObject *l, *item;
	char **list;
	if (!PyArg_ParseTuple(args, "O!iO:gtk_clist_insert", &PyGtk_Type, &cl,
			&r, &l))
		return NULL;
	if (!PySequence_Check(l)) {
		PyErr_SetString(PyExc_TypeError, "argument not a sequence");
		return NULL;
	}
	col = GTK_CLIST(PyGtk_Get(cl))->columns;
	if (PySequence_Length(l) < col) {
		PyErr_SetString(PyExc_TypeError, "sequnce too short");
		return NULL;
	}
	list = malloc(sizeof(char *) * col);
	for (i = 0; i < col; i++) {
		item = PySequence_GetItem(l, i);
		if (!PyString_Check(item)) {
			PyErr_SetString(PyExc_TypeError,
				"sequence item not a string");
			free(list);
			return NULL;
		}
		list[i] = PyString_AsString(item);
	}
	gtk_clist_insert(GTK_CLIST(PyGtk_Get(cl)), r, list);
	free(list);
	Py_INCREF(Py_None);
	return Py_None;
}
static PyObject *gtk__clist_set_row_data(PyObject *self, PyObject *args) {
        PyGtk_Object *obj;
	int row;
	PyObject *data;
	if (!PyArg_ParseTuple(args, "O!iO:gtk_clist_set_row_data", &PyGtk_Type,
			      &obj, &row, &data))
	    return NULL;
	Py_INCREF(data);
	gtk_clist_set_row_data_full(GTK_CLIST(PyGtk_Get(obj)), row, data,
				    (GtkDestroyNotify)PyGtk_DestroyNotify);
	Py_INCREF(Py_None);
	return Py_None;
}
%}
%native(gtk_clist_append) PyObject *gtk__clist_append(PyObject *s,PyObject *a);
%native(gtk_clist_insert) PyObject *gtk__clist_insert(PyObject *s,PyObject *a);
void gtk_clist_remove(GtkCList *cl, int row);
void gtk_clist_select_row(GtkCList *cl, int r, int c);
void gtk_clist_unselect_row(GtkCList *cl, int r, int c);
void gtk_clist_clear(GtkCList *cl);
%native(gtk_clist_set_row_data) PyObject *gtk__clist_set_row_data(PyObject *s,
								  PyObject *a);
PyObject *gtk_clist_get_row_data(GtkCList *cl, int row);
int gtk_clist_find_row_from_data(GtkCList *cl, PyObject *data);

int GTK_IS_COLOR_SELECTION(GtkObject *o);
int GTK_IS_COLOR_SELECTION_DIALOG(GtkObject *o);
GtkWidget *gtk_color_selection_new();
void gtk_color_selection_set_update_policy(GtkColorSelection *c, int p);
void gtk_color_selection_set_opacity(GtkColorSelection *c, int uo);
void gtk_color_selection_set_color(GtkColorSelection *c, gdouble *color);
void gtk_color_selection_get_color(GtkColorSelection *c, gdouble *ocolor);
%{
#define gtk_color_selection_dialog_get_colorsel(d)      ((d)->colorsel)
#define gtk_color_selection_dialog_get_main_vbox(d)     ((d)->main_vbox)
#define gtk_color_selection_dialog_get_ok_button(d)     ((d)->ok_button)
#define gtk_color_selection_dialog_get_reset_button(d)  ((d)->reset_button)
#define gtk_color_selection_dialog_get_cancel_button(d) ((d)->cancel_button)
#define gtk_color_selection_dialog_get_help_button(d)   ((d)->help_button)
%}
GtkWidget *gtk_color_selection_dialog_new(char *title);
GtkWidget *gtk_color_selection_dialog_get_colorsel(GtkColorSelectionDialog *c);
GtkWidget *gtk_color_selection_dialog_get_main_vbox(GtkColorSelectionDialog *c);
GtkWidget *gtk_color_selection_dialog_get_ok_button(GtkColorSelectionDialog *c);
GtkWidget *gtk_color_selection_dialog_get_reset_button(GtkColorSelectionDialog *c);
GtkWidget *gtk_color_selection_dialog_get_cancel_button(GtkColorSelectionDialog *c);
GtkWidget *gtk_color_selection_dialog_get_help_button(GtkColorSelectionDialog *c);
int GTK_IS_COMBO(GtkObject *o);
%{
#define gtk_combo_get_entry(c) ((c)->entry)
#define gtk_combo_get_list(c)  ((c)->list)
%}
GtkWidget *gtk_combo_new();
void gtk_combo_set_value_in_list(GtkCombo *c, int val, int ok_if_empty);
void gtk_combo_set_use_arrows(GtkCombo *c, int val);
void gtk_combo_set_use_arrows_always(GtkCombo *c, int val);
void gtk_combo_set_case_sensitive(GtkCombo *c, int val);
void gtk_combo_set_item_string(GtkCombo *c, GtkItem *i, char *val);
// gtk_combo_set_popdown_strings
GtkWidget *gtk_combo_get_entry(GtkCombo *c);
GtkWidget *gtk_combo_get_list(GtkCombo *c);
void gtk_combo_disable_activate(GtkCombo *c);
int GTK_IS_CONTAINER(GtkObject *o);
void gtk_container_border_width(GtkContainer *c, int bw);
void gtk_container_add(GtkContainer *c, GtkWidget *w);
void gtk_container_remove(GtkContainer *c, GtkWidget *w);
void gtk_container_disable_resize(GtkContainer *c);
void gtk_container_enable_resize(GtkContainer *c);
void gtk_container_block_resize(GtkContainer *c);
void gtk_container_unblock_resize(GtkContainer *c);
int gtk_container_need_resize(GtkContainer *c);
// gtk_container_foreach
int gtk_container_focus(GtkContainer *c, int d);
GList *gtk_container_children(GtkContainer *c);
int GTK_IS_CURVE(GtkObject *o);
GtkWidget *gtk_curve_new();
void gtk_curve_reset(GtkCurve *c);
void gtk_curve_set_gamma(GtkCurve *c, double g);
void gtk_curve_set_range(GtkCurve *c, double min_x, double max_x,
                                      double min_y, double max_y);
// gtk_curve_get_vector
// gtk_curve_set_vector
void gtk_curve_set_curve_type(GtkCurve *c, int t);
int GTK_IS_DATA(GtkObject *o);
%{
#define gtk_dialog_get_vbox(d) ((d)->vbox)
#define gtk_dialog_get_action_area(d) ((d)->action_area)
%}
int GTK_IS_DIALOG(GtkObject *o);
GtkWidget *gtk_dialog_new();
GtkWidget *gtk_dialog_get_vbox(GtkDialog *d);
GtkWidget *gtk_dialog_get_action_area(GtkDialog *d);
int GTK_IS_DRAWING_AREA(GtkObject *o);
GtkWidget *gtk_drawing_area_new();
void gtk_drawing_area_size(GtkDrawingArea *d, int w, int h);
int GTK_IS_EDITABLE(GtkObject *o);
void gtk_editable_select_region(GtkEditable *e, int start, int end);
%{
static PyObject *gtk__editable_insert_text(PyObject *self, PyObject *args) {
	PyGtk_Object *o;
	char *text;
	int len, pos;
	if (!PyArg_ParseTuple(args, "O!s#:gtk_editable_insert_text",
			&PyGtk_Type, &o, &text, &len))
		return NULL;
	gtk_editable_insert_text(GTK_EDITABLE(PyGtk_Get(o)), text, len, &pos);
	return PyInt_FromLong(pos);
}
%}
%native(gtk_editable_insert_text) PyObject *gtk__editable_insert_text(
	PyObject *self, PyObject *args);
void gtk_editable_delete_text(GtkEditable *e, int start, int end);
char *gtk_editable_get_chars(GtkEditable *e, int start, int end);
void gtk_editable_cut_clipboard(GtkEditable *e, int time);
void gtk_editable_copy_clipboard(GtkEditable *e, int time);
void gtk_editable_paste_clipboard(GtkEditable *e, int time);
void gtk_editable_claim_selection(GtkEditable *e, int claim, int time);
void gtk_editable_delete_selection(GtkEditable *e);
void gtk_editable_changed(GtkEditable *e);
int GTK_IS_ENTRY(GtkObject *o);
GtkWidget *gtk_entry_new();
GtkWidget *gtk_entry_new_with_max_length(int max);
void gtk_entry_set_text(GtkEntry *e, char *t);
void gtk_entry_append_text(GtkEntry *e, char *t);
void gtk_entry_prepend_text(GtkEntry *e, char *t);
void gtk_entry_set_position(GtkEntry *e, int p);
char *gtk_entry_get_text(GtkEntry *e);
void gtk_entry_select_region(GtkEntry *e, int start, int stop);
void gtk_entry_set_visibility(GtkEntry *e, int visible);
void gtk_entry_set_editable(GtkEntry *e, int editable);
void gtk_entry_set_max_length(GtkEntry *e, int max);
int GTK_IS_EVENT_BOX(GtkObject *o);
GtkWidget *gtk_event_box_new();

%{
#define gtk_file_selection_get_dir_list(f)  ((f)->dir_list)
#define gtk_file_selection_get_file_list(f) ((f)->file_list)
#define gtk_file_selection_get_selection_entry(f) ((f)->selection_entry)
#define gtk_file_selection_get_selection_text(f)  ((f)->selection_text)
#define gtk_file_selection_get_main_vbox(f) ((f)->main_vbox)
#define gtk_file_selection_get_ok_button(f) ((f)->ok_button)
#define gtk_file_selection_get_cancel_button(f) ((f)->cancel_button)
#define gtk_file_selection_get_help_button(f) ((f)->help_button)
%}
GtkWidget *gtk_file_selection_get_dir_list(GtkFileSelection *f);
GtkWidget *gtk_file_selection_get_file_list(GtkFileSelection *f);
GtkWidget *gtk_file_selection_get_selection_entry(GtkFileSelection *f);
GtkWidget *gtk_file_selection_get_selection_text(GtkFileSelection *f);
GtkWidget *gtk_file_selection_get_main_vbox(GtkFileSelection *f);
GtkWidget *gtk_file_selection_get_ok_button(GtkFileSelection *f);
GtkWidget *gtk_file_selection_get_cancel_button(GtkFileSelection *f);
GtkWidget *gtk_file_selection_get_help_button(GtkFileSelection *f);

int GTK_IS_FILE_SELECTION(GtkObject *o);
GtkWidget *gtk_file_selection_new(char *title);
void gtk_file_selection_set_filename(GtkFileSelection *f, char *file);
char *gtk_file_selection_get_filename(GtkFileSelection *f);
int GTK_IS_FIXED(GtkObject *o);
GtkWidget *gtk_fixed_new();
void gtk_fixed_put(GtkFixed *fixed, GtkWidget *w, int x, int y);
void gtk_fixed_move(GtkFixed *fixed, GtkWidget *w, int x, int y);
int GTK_IS_FRAME(GtkObject *o);
GtkWidget *gtk_frame_new(char *label);
void gtk_frame_set_label(GtkFrame *f, char *label);
void gtk_frame_set_label_align(GtkFrame *f, double xa, double ya);
void gtk_frame_set_shadow_type(GtkFrame *f, int st);
int GTK_IS_GAMMA_CURVE(GtkObject *o);
GtkWidget *gtk_gamma_curve_new();
int GTK_IS_HANDLE_BOX(GtkObject *o);
GtkWidget *gtk_handle_box_new();
int GTK_IS_HBOX(GtkObject *o);
GtkWidget *gtk_hbox_new(int homogeneous, int spacing);
int GTK_IS_HBUTTON_BOX(GtkObject *o);
GtkWidget *gtk_hbutton_box_new();
int gtk_hbutton_box_get_spacing_default();
int gtk_hbutton_box_get_layout_default();
void gtk_hbutton_box_set_spacing_default(int s);
void gtk_hbutton_box_set_layout_default(int l);
int GTK_IS_HPANED(GtkObject *o);
GtkWidget *gtk_hpaned_new();
int GTK_IS_HRULER(GtkObject *o);
GtkWidget *gtk_hruler_new();
int GTK_IS_HSCALE(GtkObject *o);
GtkWidget *gtk_hscale_new(GtkAdjustment *adjustment = NULL);
int GTK_IS_HSCROLLBAR(GtkObject *o);
GtkWidget *gtk_hscrollbar_new(GtkAdjustment *adjustment = NULL);
int GTK_IS_HSEPARATOR(GtkObject *o);
GtkWidget *gtk_hseparator_new();
int GTK_IS_IMAGE(GtkObject *o);
// GtkWidget *gtk_image_new(GdkImage *val, GdkBitmap *mask);
// void gtk_image_set(GtkImage *i, GdkImage *val, GdkBitmap *mask);
// void gtk_image_get(GtkImage *i, GdkImage **val, GdkBitmap **mask);
int GTK_IS_INPUT_DIALOG(GtkObject *o);
GtkWidget *gtk_input_dialog_new();
int GTK_IS_ITEM(GtkObject *o);
void gtk_item_select(GtkItem *i);
void gtk_item_deselect(GtkItem *i);
void gtk_item_toggle(GtkItem *i);
int GTK_IS_LABEL(GtkObject *o);
GtkWidget *gtk_label_new(char *str);
void gtk_label_set(GtkLabel *l, char *str);
void gtk_label_get(GtkLabel *l, char **outstr);
void gtk_label_set_justify(GtkLabel *l, int just);
%{
#define gtk_list_get_selection(l) ((l)->selection)
%}
int GTK_IS_LIST(GtkObject *o);
GtkWidget *gtk_list_new();
GList *gtk_list_get_selection(GtkList *l);
void gtk_list_insert_items(GtkList *l, GList *items, int position);
void gtk_list_append_items(GtkList *l, GList *items);
void gtk_list_prepend_items(GtkList *l, GList *items);
void gtk_list_remove_items(GtkList *l, GList *items);
void gtk_list_clear_items(GtkList *l, int start, int end);
void gtk_list_select_item(GtkList *l, int item);
void gtk_list_unselect_item(GtkList *l, int item);
void gtk_list_select_child(GtkList *l, GtkWidget *child);
void gtk_list_unselect_child(GtkList *l, GtkWidget *child);
int gtk_list_child_position(GtkList *l, GtkWidget *child);
void gtk_list_set_selection_mode(GtkList *l, int mode);
int GTK_IS_LIST_ITEM(GtkObject *o);
GtkWidget *gtk_list_item_new();
GtkWidget *gtk_list_item_new_with_label(char *label);
void gtk_list_item_select(GtkListItem *i);
void gtk_list_item_deselect(GtkListItem *i);
int GTK_IS_MENU(GtkObject *o);
GtkWidget *gtk_menu_new();
void gtk_menu_append(GtkMenu *m, GtkWidget *child);
void gtk_menu_prepend(GtkMenu *m, GtkWidget *child);
void gtk_menu_insert(GtkMenu *m, GtkWidget *child, int pos);
%{
static void PyGtk_MenuPosition(GtkMenu *menu, int *x, int *y, PyObject *func) {
    PyObject *ret;
    ret = PyObject_CallFunction(func, "(O)", PyGtk_New(GTK_OBJECT(menu)));
    if (ret == NULL || !PyArg_ParseTuple(ret, "ii", x, y)) {
        PyErr_Clear();
        fprintf(stderr, "Ignoring Exception\n");
        *x = 0;
        *y = 0;
    } else
        Py_DECREF(ret);
}
static PyObject *gtk__menu_popup(PyObject *self, PyObject *args) {
    PyGtk_Object *m, *pms, *pmi;
    PyObject *func;
    int button, time;
    if (!PyArg_ParseTuple(args, "O!O!O!Oii:gtk_menu_item", &PyGtk_Type, &m,
            &PyGtk_Type, &pms, &PyGtk_Type, &pmi, &func, &button, &time))
        return NULL;
    if (!PyCallable_Check(func)) {
        PyErr_SetString(PyExc_TypeError, "forth argument not callable");
        return NULL;
    }
    Py_INCREF(func);
    gtk_menu_popup(GTK_MENU(PyGtk_Get(m)), GTK_WIDGET(PyGtk_Get(pms)),
        GTK_WIDGET(PyGtk_Get(pmi)), (GtkMenuPositionFunc)PyGtk_MenuPosition,
        func, button, time);
    Py_INCREF(Py_None);
    return Py_None;
}
%}
%native(gtk_menu_popup) PyObject *gtk__menu_popup(PyObject *s,PyObject *args);
void gtk_menu_popdown(GtkMenu *m);
GtkWidget *gtk_menu_get_active(GtkMenu *m);
void gtk_menu_set_active(GtkMenu *m, int pos);
void gtk_menu_set_accelerator_table(GtkMenu *m, GtkAcceleratorTable *table);
GtkWidget *gtk_menu_get_attach_widget(GtkMenu *m);
void gtk_menu_detach(GtkMenu *m);
int GTK_IS_MENU_BAR(GtkObject *o);
GtkWidget *gtk_menu_bar_new();
void gtk_menu_bar_append(GtkMenuBar *m, GtkWidget *child);
void gtk_menu_bar_prepend(GtkMenuBar *m, GtkWidget *child);
void gtk_menu_bar_insert(GtkMenuBar *m, GtkWidget *child, int pos);
int GTK_IS_MENU_ITEM(GtkObject *o);
GtkWidget *gtk_menu_item_new();
GtkWidget *gtk_menu_item_new_with_label(char *label);
void gtk_menu_item_set_submenu(GtkMenuItem *m, GtkWidget *submenu);
void gtk_menu_item_remove_submenu(GtkMenuItem *m);
void gtk_menu_item_set_placement(GtkMenuItem *m, int placement);
void gtk_menu_item_accelerator_size(GtkMenuItem *m);
void gtk_menu_item_accelerator_text(GtkMenuItem *m, char *buffer);
void gtk_menu_item_configure(GtkMenuItem *m, int show_toggle, int show_sub);
void gtk_menu_item_select(GtkMenuItem *m);
void gtk_menu_item_deselect(GtkMenuItem *m);
void gtk_menu_item_activate(GtkMenuItem *m);
void gtk_menu_item_right_justify(GtkMenuItem *m);
int GTK_IS_MENU_SHELL(GtkObject *o);
void gtk_menu_shell_append(GtkMenuShell *m, GtkWidget *child);
void gtk_menu_shell_prepend(GtkMenuShell *m, GtkWidget *child);
void gtk_menu_shell_insert(GtkMenuShell *m, GtkWidget *child, int pos);
void gtk_menu_shell_deactivate(GtkMenuShell *m);
int GTK_IS_MISC(GtkObject *o);
void gtk_misc_set_alignment(GtkMisc *m, double xalign, double yalign);
void gtk_misc_set_padding(GtkMisc *m, int xpad, int ypad);
int GTK_IS_NOTEBOOK(GtkObject *o);
GtkWidget *gtk_notebook_new();
void gtk_notebook_append_page(GtkNotebook *n, GtkWidget *c, GtkWidget *tab);
void gtk_notebook_append_page_menu(GtkNotebook *n, GtkWidget *c,
	GtkWidget *tab, GtkWidget *menu_label);
void gtk_notebook_prepend_page(GtkNotebook *n, GtkWidget *c, GtkWidget *tab);
void gtk_notebook_prepend_page_menu(GtkNotebook *n, GtkWidget *c,
	GtkWidget *tab, GtkWidget *menu_label);
void gtk_notebook_insert_page(GtkNotebook *n, GtkWidget *c, GtkWidget *tab,
	int pos);
void gtk_notebook_insert_page_menu(GtkNotebook *n, GtkWidget *c,
	GtkWidget *tab, GtkWidget *menu_label, int pos);
void gtk_notebook_remove_page(GtkNotebook *n, int pos);
int gtk_notebook_current_page(GtkNotebook *n);
void gtk_notebook_set_page(GtkNotebook *n, int pos);
void gtk_notebook_next_page(GtkNotebook *n);
void gtk_notebook_prev_page(GtkNotebook *n);
void gtk_notebook_set_tab_pos(GtkNotebook *n, int pos);
void gtk_notebook_set_show_tabs(GtkNotebook *n, int show);
void gtk_notebook_set_show_border(GtkNotebook *n, int show);
void gtk_notebook_set_scrollable(GtkNotebook *n, int scrollable);
void gtk_notebook_set_tab_border(GtkNotebook *n, int border_width);
void gtk_notebook_popup_enable(GtkNotebook *n);
void gtk_notebook_popup_disable(GtkNotebook *n);
int GTK_IS_OPTION_MENU(GtkObject *o);
GtkWidget *gtk_option_menu_new();
GtkWidget *gtk_option_menu_get_menu(GtkOptionMenu *o);
void gtk_option_menu_set_menu(GtkOptionMenu *o, GtkWidget *m);
void gtk_option_menu_remove_menu(GtkOptionMenu *o);
void gtk_option_menu_set_history(GtkOptionMenu *o, int index);
int GTK_IS_PANED(GtkObject *o);
void gtk_paned_add1(GtkPaned *paned, GtkWidget *child);
void gtk_paned_add2(GtkPaned *paned, GtkWidget *child);
void gtk_paned_handle_size(GtkPaned *paned, int size);
void gtk_paned_gutter_size(GtkPaned *paned, int size);
int GTK_IS_PIXMAP(GtkObject *o);
%{
static PyObject *gtk__pixmap_new_from_xpm(PyObject *self, PyObject *args) {
    PyGtk_Object *win;
    PyGdkColor_Object *tc = NULL;
    char *name;
    GdkColor *bg;
    GdkPixmap *pm;
    GdkBitmap *bm;
    GtkStyle *style;
    PyObject *ret;

    if (!PyArg_ParseTuple(args, "O!s|O!:gtk_pixmap_new_from_xpm", &PyGtk_Type,
            &win, &name, &PyGdkColor_Type, &tc))
        return  NULL;
    style = gtk_widget_get_style(GTK_WIDGET(PyGtk_Get(win)));
    if (tc == NULL)
        bg = &style->bg[GTK_STATE_NORMAL];
    else
        bg = PyGdkColor_Get(tc);
    pm = gdk_pixmap_create_from_xpm(GTK_WIDGET(PyGtk_Get(win))->window, &bm,
        bg, name);
    ret = PyGtk_New(GTK_OBJECT(gtk_pixmap_new(pm, bm)));
    /* gtk_pixmap_new ref's the pixmap and mask, so we can unref out vars */
    gdk_pixmap_unref(pm);
    gdk_bitmap_unref(bm);
    return ret;
}
%}
%native(gtk_pixmap_new_from_xpm) PyObject *gtk__pixmap_new_from_xpm(PyObject
    *self, PyObject *args);
// GtkWidget *gtk_pixmap_new(GdkPixmap *pixmap, GdkBitmap *mask);
// void gtk_pixmap_set(GtkPixmap *p, GdkPixmap *val, GdkBitmap *mask);
// void gtk_pixmap_get(GtkPixmap *p, GdkPixmap **val, GdkBitmap **mask);
int GTK_IS_PREVIEW(GtkObject *o);
void gtk_preview_uninit();
GtkWidget *gtk_preview_new(int preview_type);
void gtk_preview_size(GtkPreview *p, int w, int h);
// gtk_preview_put
// gtk_preview_put_row
// gtk_preview_draw_row
void gtk_preview_set_expand(GtkPreview *p, int expand);
void gtk_preview_set_gamma(double gamma);
void gtk_preview_set_color_cube(unsigned int nred, unsigned int ngreen,
                                unsigned int nblue, unsigned int ngrey);
void gtk_preview_set_install_cmap(int install);
void gtk_preview_set_reserved(int res);
// gtk_preview_get_visual
// gtk_preview_get_cmap
// gtk_preview_get_info
int GTK_IS_PROGRESS_BAR(GtkObject *o);
GtkWidget *gtk_progress_bar_new();
void gtk_progress_bar_update(GtkProgressBar *p, double percent);
int GTK_IS_RADIO_BUTTON(GtkObject *o);
%{
static PyObject *gtk__radio_button_new(PyObject *self, PyObject *args) {
    PyGtk_Object *g = NULL;
    GSList *l = NULL;
    if (!PyArg_ParseTuple(args, "|O!:gtk_radio_button_new", &PyGtk_Type, &g))
        return NULL;
    if (g)
        l = gtk_radio_button_group(GTK_RADIO_BUTTON(PyGtk_Get(g)));
    return PyGtk_New(GTK_OBJECT(gtk_radio_button_new(l)));
}
static PyObject *gtk__radio_button_new_with_label(PyObject *s,PyObject *args){
    PyGtk_Object *g = NULL;
    char *label = "";
    GSList *l = NULL;
    if (!PyArg_ParseTuple(args, "|Os:gtk_radio_button_new_with_label",
            &g, &label))
        return NULL;
    if (g != NULL)
        if (!PyGtk_Check(g) && (PyObject *)g != Py_None) {
            PyErr_SetString(PyExc_TypeError,
                "first argument not a GtkObject or None");
            return NULL;
        }
    if ((PyObject *)g == Py_None) g = NULL;
    if (g)
        l = gtk_radio_button_group(GTK_RADIO_BUTTON(PyGtk_Get(g)));
    return PyGtk_New(GTK_OBJECT(gtk_radio_button_new_with_label(l, label)));
}
%}
%native(gtk_radio_button_new) PyObject *gtk__radio_button_new(PyObject *self,
    PyObject *args);
%native(gtk_radio_button_new_with_label) PyObject *
    gtk__radio_button_new_with_label(PyObject *s,PyObject *args);
int GTK_IS_RADIO_MENU_ITEM(GtkObject *o);
%{
static PyObject *gtk__radio_menu_item_new(PyObject *self, PyObject *args) {
    PyGtk_Object *g = NULL;
    GSList *l = NULL;
    if (!PyArg_ParseTuple(args, "|O!:gtk_radio_menu_item_new",&PyGtk_Type, &g))
        return NULL;
    if (g)
        l = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(PyGtk_Get(g)));
    return PyGtk_New(GTK_OBJECT(gtk_radio_menu_item_new(l)));
}
static PyObject *gtk__radio_menu_item_new_with_label(PyObject *s,PyObject *a) {
    PyGtk_Object *g = NULL;
    char *label = "";
    GSList *l = NULL;
    if (!PyArg_ParseTuple(a, "|Os:gtk_radio_menu_item_new_with_label",
            &g, &label))
        return NULL;
    if (g != NULL)
        if (!PyGtk_Check(g) && (PyObject *)g != Py_None) {
            PyErr_SetString(PyExc_TypeError,
                "first argument not a GtkObject or None");
            return NULL;
        }
    if ((PyObject *)g == Py_None) g = NULL;
    if (g)
        l = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(PyGtk_Get(g)));
    return PyGtk_New(GTK_OBJECT(gtk_radio_menu_item_new_with_label(l, label)));
}
%}
%native(gtk_radio_menu_item_new) PyObject *gtk__radio_menu_item_new(PyObject
    *self, PyObject *args);
%native(gtk_radio_menu_item_new_with_label) PyObject *
    gtk__radio_menu_item_new_with_label(PyObject *s,PyObject *a);
int GTK_IS_RANGE(GtkObject *o);
GtkAdjustment *gtk_range_get_adjustment(GtkRange *r);
void gtk_range_set_update_policy(GtkRange *r, int update);
void gtk_range_set_adjustment(GtkRange *r, GtkAdjustment *a);
void gtk_range_draw_background(GtkRange *r);
void gtk_range_draw_trough(GtkRange *r);
void gtk_range_draw_slider(GtkRange *r);
void gtk_range_draw_step_forw(GtkRange *r);
void gtk_range_draw_step_back(GtkRange *r);
void gtk_range_slider_update(GtkRange *r);
// These seven should not be available directly -- they are for descendants
//void gtk_range_trough_click(GtkRange *r, int x, int y, double *jump_perc);
//void gtk_range_default_hslider_update(GtkRange *r);
//void gtk_range_default_vslider_update(GtkRange *r);
//void gtk_range_default_htrough_click(GtkRange *r, int x, int y);
//void gtk_range_default_vtrough_click(GtkRange *r, int x, int y);
//void gtk_range_default_hmotion(GtkRange *r, int x, int y);
//void gtk_range_default_vmotion(GtkRange *r, int x, int y);
// This one disapeared with 0.99.5
//double gtk_range_calc_value(GtkRange *r, int pos);
int GTK_IS_RULER(GtkObject *o);
void gtk_ruler_set_metric(GtkRuler *r, int metric);
void gtk_ruler_set_range(GtkRuler *r, double lower, double upper, double pos,
                                                             double max_size);
void gtk_ruler_draw_ticks(GtkRuler *r);
void gtk_ruler_draw_pos(GtkRuler *r);
int GTK_IS_SCALE(GtkObject *o);
void gtk_scale_set_digits(GtkScale *s, int digits);
void gtk_scale_set_draw_value(GtkScale *s, int value);
void gtk_scale_set_value_pos(GtkScale *s, int pos);
int gtk_scale_value_width(GtkScale *s);
void gtk_scale_draw_value(GtkScale *s);
int GTK_IS_SCROLLBAR(GtkObject *o);
int GTK_IS_SCROLLED_WINDOW(GtkObject *o);
GtkWidget *gtk_scrolled_window_new(GtkAdjustment *h = NULL, GtkAdjustment *v = NULL);
GtkAdjustment *gtk_scrolled_window_get_hadjustment(GtkScrolledWindow *s);
GtkAdjustment *gtk_scrolled_window_get_vadjustment(GtkScrolledWindow *s);
void gtk_scrolled_window_set_policy(GtkScrolledWindow *s, int hp, int vp);
int GTK_IS_SEPARATOR(GtkObject *o);
int GTK_IS_SPIN_BUTTON(GtkObject *o);
%{
#define gtk_spin_button_new_no_adj(c,d) gtk_spin_button_new(NULL,c,d)
%}
GtkWidget *gtk_spin_button_new(GtkAdjustment *a, double climb, int digits);
GtkWidget *gtk_spin_button_new_no_adj(double climb, int digits);
void gtk_spin_button_set_adjustment(GtkSpinButton *s, GtkAdjustment *a);
GtkAdjustment *gtk_spin_button_get_adjustment(GtkSpinButton *s);
void gtk_spin_button_set_digits(GtkSpinButton *s, int digits);
double gtk_spin_button_get_value_as_float(GtkSpinButton *s);
int gtk_spin_button_get_value_as_int(GtkSpinButton *s);
void gtk_spin_button_set_value(GtkSpinButton *s, double value);
void gtk_spin_button_set_update_policy(GtkSpinButton *s, int policy);
void gtk_spin_button_set_numeric(GtkSpinButton *s, int numeric);
void gtk_spin_button_spin(GtkSpinButton *s, int direction, double step);
int GTK_IS_STATUSBAR(GtkObject *o);
GtkWidget *gtk_statusbar_new();
int gtk_statusbar_get_context_id(GtkStatusbar *s, char *context_description);
int gtk_statusbar_push(GtkStatusbar *s, int statusid, char *text);
void gtk_statusbar_pop(GtkStatusbar *s, int statusid);
void gtk_statusbar_remove(GtkStatusbar *s, int context_id, int message_id);
int GTK_IS_TABLE(GtkObject *o);
GtkWidget *gtk_table_new(int rows, int cols, int homogeneous);
void gtk_table_attach(GtkTable *t, GtkWidget *child, int la, int ra, int ta,
                                     int ba, int xo, int yo, int xp, int yp);
void gtk_table_attach_defaults(GtkTable *t, GtkWidget *child, int la, int ra,
                                                              int ta, int ba);
void gtk_table_set_row_spacing(GtkTable *t, int row, int spacing);
void gtk_table_set_col_spacing(GtkTable *t, int col, int spacing);
void gtk_table_set_row_spacings(GtkTable *t, int spacing);
void gtk_table_set_col_spacings(GtkTable *t, int spacing);
%{
#define gtk_text_get_hadjustment(t) ((t)->hadj)
#define gtk_text_get_vadjustment(t) ((t)->vadj)
%}
int GTK_IS_TEXT(GtkObject *o);
GtkWidget *gtk_text_new(GtkAdjustment *ha = NULL, GtkAdjustment *va = NULL);
void gtk_text_set_editable(GtkText *t, int editable);
void gtk_text_set_word_wrap(GtkText *t, int word_wrap);
void gtk_text_set_adjustments(GtkText *t, GtkAdjustment *ha,GtkAdjustment *va);
GtkAdjustment *gtk_text_get_hadjustment(GtkText *t);
GtkAdjustment *gtk_text_get_vadjustment(GtkText *t);
void gtk_text_set_point(GtkText *t, unsigned int index);
unsigned int gtk_text_get_point(GtkText *t);
unsigned int gtk_text_get_length(GtkText *t);
void gtk_text_freeze(GtkText *t);
void gtk_text_thaw(GtkText *t);
void gtk_text_insert(GtkText *t, GdkFont *f, GdkColor *fore, GdkColor *back, char *chars, int length);
int gtk_text_backward_delete(GtkText *t, unsigned int nchars);
int gtk_text_forward_delete(GtkText *t, unsigned int nchars);
%{
static PyObject *gtk__text_insert_defaults(PyObject *self, PyObject *args) {
    PyGtk_Object *txt;
    char *chars;
    int l;
    GtkStyle *style;
    if (!PyArg_ParseTuple(args, "O!s#:gtk_text_insert_defaults", &PyGtk_Type,
                                     &txt, &chars, &l))
        return NULL;
    style = gtk_widget_get_style(GTK_WIDGET(PyGtk_Get(txt)));
    gtk_text_insert(GTK_TEXT(PyGtk_Get(txt)), NULL,
        &style->fg[GTK_STATE_NORMAL], NULL, chars, l);
    Py_INCREF(Py_None);
    return Py_None;
}
%}
%native(gtk_text_insert_defaults) PyObject *gtk__text_insert_defaults(
                                           PyObject *self, PyObjject *args);
int GTK_IS_TIPS_QUERY(GtkObject *o);
GtkWidget *gtk_tips_query_new();
void gtk_tips_query_start_query(GtkTipsQuery *tq);
void gtk_tips_query_stop_query(GtkTipsQuery *qt);
void gtk_tips_query_set_caller(GtkTipsQuery *qt, GtkWidget *w);
void gtk_tips_query_set_labels(GtkTipsQuery *qt, char *inactive, char *no_tip);
%{
#define gtk_toggle_button_get_active(tb) ((tb)->active)
%}
int GTK_IS_TOGGLE_BUTTON(GtkObject *o);
GtkWidget *gtk_toggle_button_new();
GtkWidget *gtk_toggle_button_new_with_label(char *label);
void gtk_toggle_button_set_mode(GtkToggleButton *t, int draw_i);
void gtk_toggle_button_set_state(GtkToggleButton *t, int s);
void gtk_toggle_button_toggled(GtkToggleButton *t);
int gtk_toggle_button_get_active(GtkToggleButton *t);
int GTK_IS_TOOLBAR(GtkObject *o);
GtkWidget *gtk_toolbar_new(int orientation, int style);
%{
static PyObject *gtk__toolbar_append_item(PyObject *self, PyObject *args) {
	PyGtk_Object *t, *icon;
	char *text, *tooltip, *tip_private;
	PyObject *callback;
	GList *tmp_list;
	if (!PyArg_ParseTuple(args, "O!zzzO!O|gtk_toolbar_append_item",
			&PyGtk_Type, &t, &text, &tooltip, &tip_private,
                        &PyGtk_Type, &icon, &callback))
		return NULL;
	if (!PyCallable_Check(callback) && callback != Py_None) {
		PyErr_SetString(PyExc_TypeError,"sixth argument not callable");
		return NULL;
	}
	Py_INCREF(callback);
        /* if you set sigfunc to NULL, no signal is connected, rather than
           the default signal handler being used */
	gtk_toolbar_append_item(GTK_TOOLBAR(PyGtk_Get(t)), text, tooltip,
		tip_private, GTK_WIDGET(PyGtk_Get(icon)), NULL, NULL);
	if (callback != Py_None) {
		tmp_list = g_list_last(gtk_container_children(GTK_CONTAINER(
			PyGtk_Get(t))));
		gtk_signal_connect(GTK_OBJECT(tmp_list->data), "clicked", NULL,
			callback);
	}
	Py_INCREF(Py_None);
	return Py_None;
}
static PyObject *gtk__toolbar_prepend_item(PyObject *self, PyObject *args) {
	PyGtk_Object *t, *icon;
	char *text, *tooltip, *tip_private;
	PyObject *callback;
	if (!PyArg_ParseTuple(args, "O!zzzO!O|gtk_toolbar_prepend_item",
			&PyGtk_Type, &t, &text, &tooltip, &tip_private,
			&PyGtk_Type, &icon, &callback))
		return NULL;
	if (!PyCallable_Check(callback) && callback != Py_None) {
		PyErr_SetString(PyExc_TypeError,"sixth argument not callable");
		return NULL;
	}
	Py_INCREF(callback);
	gtk_toolbar_prepend_item(GTK_TOOLBAR(PyGtk_Get(t)), text, tooltip,
		tip_private, GTK_WIDGET(PyGtk_Get(icon)), NULL, NULL);
	if (callback != Py_None)
		gtk_signal_connect(GTK_OBJECT(gtk_container_children(
			GTK_CONTAINER(PyGtk_Get(t)))->data), "clicked", NULL,
			callback);
	Py_INCREF(Py_None);
	return Py_None;
}
static PyObject *gtk__toolbar_insert_item(PyObject *self, PyObject *args) {
	PyGtk_Object *t, *icon;
	char *text, *tooltip, *tip_private;
	PyObject *callback;
	int pos;
	GList *tmp_list;
	if (!PyArg_ParseTuple(args, "O!zzzO!Oi|gtk_toolbar_insert_item",
			&PyGtk_Type, &t, &text, &tooltip, &tip_private,
			&PyGtk_Type, &icon, &callback, &pos))
		return NULL;
	if (!PyCallable_Check(callback) && callback != Py_None) {
		PyErr_SetString(PyExc_TypeError,"sixth argument not callable");
		return NULL;
	}
	Py_INCREF(callback);
	gtk_toolbar_insert_item(GTK_TOOLBAR(PyGtk_Get(t)), text, tooltip,
		tip_private, GTK_WIDGET(PyGtk_Get(icon)), NULL, NULL, pos);
	if (callback != Py_None) {
		tmp_list = g_list_nth(gtk_container_children(GTK_CONTAINER(
			PyGtk_Get(t))), pos);
		gtk_signal_connect(GTK_OBJECT(tmp_list->data), "clicked", NULL,
			callback);
	}
	Py_INCREF(Py_None);
	return Py_None;
}
%}
%native(gtk_toolbar_append_item) PyObject *gtk__toolbar_append_item(
	PyObject *self, PyObject *args);
%native(gtk_toolbar_prepend_item) PyObject *gtk__toolbar_prepend_item(
	PyObject *self, PyObject *args);
%native(gtk_toolbar_insert_item) PyObject *gtk__toolbar_insert_item(
	PyObject *self, PyObject *args);
void gtk_toolbar_append_space(GtkToolbar *t);
void gtk_toolbar_prepend_space(GtkToolbar *t);
void gtk_toolbar_insert_space(GtkToolbar *t, int pos);
void gtk_toolbar_append_widget(GtkToolbar *t, GtkWidget *w, char *tooltip,
	char *tip_private);
void gtk_toolbar_prepend_widget(GtkToolbar *t, GtkWidget *w, char *tooltip,
	char *tip_private);
void gtk_toolbar_insert_widget(GtkToolbar *t, GtkWidget *w, char *tooltip,
	char *tip_private, int pos);
void gtk_toolbar_set_orientation(GtkToolbar *t, int orientation);
void gtk_toolbar_set_style(GtkToolbar *t, int style);
void gtk_toolbar_set_space_size(GtkToolbar *t, int size);
void gtk_toolbar_set_tooltips(GtkToolbar *t, int enable);
%{
#define gtk_tree_get_selection(t) ((t)->selection)
%}
int GTK_IS_TREE(GtkObject *o);
GtkWidget *gtk_tree_new();
GList *gtk_tree_get_selection(GtkTree *t);
void gtk_tree_append(GtkTree *t, GtkWidget *child);
void gtk_tree_prepend(GtkTree *t, GtkWidget *child);
void gtk_tree_insert(GtkTree *t, GtkWidget *child, int pos);
// void gtk_tree_remove_item(GtkTree *t, GtkWidget *child);
void gtk_tree_remove_items(GtkTree *t, GList *items);
void gtk_tree_clear_items(GtkTree *t, int start, int end);
void gtk_tree_select_item(GtkTree *t, int item);
void gtk_tree_unselect_item(GtkTree *t, int item);
void gtk_tree_select_child(GtkTree *t, GtkWidget *c);
void gtk_tree_unselect_child(GtkTree *t, GtkWidget *c);
int gtk_tree_child_position(GtkTree *t, GtkWidget *c);
void gtk_tree_set_selection_mode(GtkTree *t, int mode);
void gtk_tree_set_view_mode(GtkTree *t, int mode);
void gtk_tree_set_view_lines(GtkTree *t, int flag);
int GTK_IS_TREE_ITEM(GtkObject *o);
GtkWidget *gtk_tree_item_new();
GtkWidget *gtk_tree_item_new_with_label(char *label);
void gtk_tree_item_set_subtree(GtkTreeItem *t, GtkWidget *st);
void gtk_tree_item_select(GtkTreeItem *t);
void gtk_tree_item_deselect(GtkTreeItem *t);
void gtk_tree_item_expand(GtkTreeItem *t);
void gtk_tree_item_collapse(GtkTreeItem *t);
void gtk_tree_item_remove_subtree(GtkTreeItem *t);
int GTK_IS_VBOX(GtkObject *o);
GtkWidget *gtk_vbox_new(int homogeneous, int spacing);
int GTK_IS_VBUTTON_BOX(GtkObject *o);
GtkWidget *gtk_vbutton_box_new();
int gtk_vbutton_box_get_spacing_default();
void gtk_vbutton_box_set_spacing_default(int s);
int gtk_vbutton_box_get_layout_default();
void gtk_vbutton_box_set_layout_default(int l);
int GTK_IS_VIEWPORT(GtkObject *o);
GtkWidget *gtk_viewport_new(GtkAdjustment *ha = NULL, GtkAdjustment *va=NULL);
GtkAdjustment *gtk_viewport_get_hadjustment(GtkViewport *v);
GtkAdjustment *gtk_viewport_get_vadjustment(GtkViewport *v);
void gtk_viewport_set_hadjustment(GtkViewport *v, GtkAdjustment *ha);
void gtk_viewport_set_vadjustment(GtkViewport *v, GtkAdjustment *va);
void gtk_viewport_set_shadow_type(GtkViewport *v, int st);
int GTK_IS_VPANED(GtkObject *o);
GtkWidget *gtk_vpaned_new();
int GTK_IS_VRULER(GtkObject *o);
GtkWidget *gtk_vruler_new();
int GTK_IS_VSCALE(GtkObject *o);
GtkWidget *gtk_vscale_new(GtkAdjustment *a = NULL);
int GTK_IS_VSCROLLBAR(GtkObject *o);
GtkWidget *gtk_vscrollbar_new(GtkAdjustment *a = NULL);
int GTK_IS_VSEPARATOR(GtkObject *o);
GtkWidget *gtk_vseparator_new();
int GTK_IS_WINDOW(GtkObject *o);
GtkWidget *gtk_window_new(int wt);
void gtk_window_set_title(GtkWindow *w, char *title);
void gtk_window_set_wmclass(GtkWindow *w, char *wmc_name, char *wmc_class);
void gtk_window_set_focus(GtkWindow *w, GtkWidget *focus);
void gtk_window_set_default(GtkWindow *w, GtkWidget *default);
void gtk_window_set_policy(GtkWindow *w, int ashrink, int agrow, int auto_s);
void gtk_window_add_accelerator_table(GtkWindow *w, GtkAcceleratorTable *t);
void gtk_window_remove_accelerator_table(GtkWindow *w, GtkAcceleratorTable *t);
void gtk_window_position(GtkWindow *w, int pos);
int gtk_window_activate_focus(GtkWindow *w);
int gtk_window_activate_default(GtkWindow *w);
%{
static PyObject *gtk__window_set_icon(PyObject *self, PyObject *args) {
	PyGtk_Object *o, *pix;
        GtkWidget *win, *pixmap;
	GdkPixmap *p;
	GdkBitmap *m;
	if (!PyArg_ParseTuple(args, "O!O!:gtk_window_set_icon", &PyGtk_Type,
			&o, &PyGtk_Type, &pix))
		return NULL;
	win = GTK_WIDGET(PyGtk_Get(o));
	pixmap = GTK_WIDGET(PyGtk_Get(pix));
	/* remove pixmap's floating ref so it is destroyed if it isn't shown */
	gtk_object_sink(GTK_OBJECT(pixmap));
	if (!GTK_IS_PIXMAP(pixmap)) {
	    PyErr_SetString(PyExc_TypeError,"2nd argument must be a pixmap");
	    return NULL;
	}
	p = gdk_pixmap_ref(GTK_PIXMAP(pixmap)->pixmap);
	m = gdk_bitmap_ref(GTK_PIXMAP(pixmap)->mask);
	gdk_window_set_icon(win->window, NULL, p, m);
	Py_INCREF(Py_None);
	return Py_None;
}
static PyObject *gtk__window_set_icon_name(PyObject *self, PyObject *args) {
	PyGtk_Object *o;
	char *name;
	if (!PyArg_ParseTuple(args, "O!s:gtk_window_set_icon_name",
			&PyGtk_Type, &o, &name))
		return NULL;
	gdk_window_set_icon_name(GTK_WIDGET(PyGtk_Get(o))->window, name);
	Py_INCREF(Py_None);
	return Py_None;
}
%}
%native(gtk_window_set_icon) PyObject *gtk__window_set_icon(
	PyObject *self, PyObject *args);
%native(gtk_window_set_icon_name) PyObject *gtk__window_set_icon_name(
	PyObject *self, PyObject *args);

// GtkAcceleratorTable Functions
GtkAcceleratorTable *gtk_accelerator_table_new();
GtkAcceleratorTable *gtk_accelerator_table_find(GtkObject *o, char *sig_name,
    unsigned char acc_key, int acc_mods);
void gtk_accelerator_table_install(GtkAcceleratorTable *t, GtkObject *o,
    char *sig_name, unsigned char acc_key, int acc_mods);
void gtk_accelerator_table_remove(GtkAcceleratorTable *t, GtkObject *o,
    char *signame);
int gtk_accelerator_table_check(GtkAcceleratorTable *t, unsigned char acc_key,
    int acc_mods);
void gtk_accelerator_table_set_mod_mask(GtkAcceleratorTable *t, int mod_mask);

// GtkTooltips Functions
GtkTooltips *gtk_tooltips_new();
void gtk_tooltips_enable(GtkTooltips *t);
void gtk_tooltips_set_delay(GtkTooltips *t, int delay);
void gtk_tooltips_set_tip(GtkTooltips *t, GtkWidget *w, char *tip, char *tip_p);
void gtk_tooltips_set_colors(GtkTooltips *t, GdkColor *b, GdkColor *f);


